/*
 Copyright (c) 2001, Loki software, inc.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice, this list
 of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright notice, this
 list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

 Neither the name of Loki software nor the names of its contributors may be used
 to endorse or promote products derived from this software without specific prior
 written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT,INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(INCLUDED_GTKMISC_H)
#define INCLUDED_GTKMISC_H

#include <string>
#include <gtk/gtkmain.h>
#include "radiant_i18n.h"

inline void process_gui ()
{
	while (gtk_events_pending()) {
		gtk_main_iteration();
	}
}

void command_connect_accelerator (const std::string& commandName);
void command_disconnect_accelerator (const std::string& commandName);
void toggle_add_accelerator (const std::string& commandName);
void toggle_remove_accelerator (const std::string& name);

typedef struct _GtkMenu GtkMenu;
typedef struct _GtkMenuItem GtkMenuItem;
typedef struct _GtkCheckMenuItem GtkCheckMenuItem;

// this also sets up the shortcut using command_connect_accelerator
GtkMenuItem
		* create_menu_item_with_mnemonic (GtkMenu *menu, const std::string& mnemonic, const std::string& commandName, const std::string& icon = "");
// this also sets up the shortcut using command_connect_accelerator
GtkCheckMenuItem* create_check_menu_item_with_mnemonic (GtkMenu* menu, const std::string& mnemonic,
		const std::string& commandName, const std::string& icon = "");

typedef struct _GtkButton GtkButton;
typedef struct _GtkToggleButton GtkToggleButton;
typedef struct _GtkToolbar GtkToolbar;

// this DOES NOT set up the shortcut using command_connect_accelerator
GtkButton* toolbar_append_button (GtkToolbar* toolbar, const std::string& description, const std::string& icon,
		const std::string& commandName);
// this DOES NOT set up the shortcut using command_connect_accelerator
GtkToggleButton* toolbar_append_toggle_button (GtkToolbar* toolbar, const std::string& description,
		const std::string& icon, const std::string& commandName);

template<typename Element> class BasicVector3;
typedef BasicVector3<float> Vector3;
bool color_dialog (GtkWidget *parent, Vector3& color, const std::string& title = _("Choose Color"));

typedef struct _GtkEntry GtkEntry;
void button_clicked_entry_browse_file (GtkWidget* widget, GtkEntry* entry);
void button_clicked_entry_browse_directory (GtkWidget* widget, GtkEntry* entry);

#endif
